Module: ComposableAgents::Cline
- Defined in:
- lib/composable_agents/cline/agent.rb
Overview
All agents from this module work with the awesome cline-rb Rubygem
Defined Under Namespace
Classes: Agent, MissingSkillError
All agents from this module work with the awesome cline-rb Rubygem
Classes: Agent, MissingSkillError